package com.googlecode.jpattern.core.command;

import com.googlecode.jpattern.core.IProvider;
import com.googlecode.jpattern.core.NullProvider;

/**
 * 
 * @author Francesco Cina'
 *
 * 27/feb/2011
 */
public abstract class ACommand extends ICommand implements Runnable {

	private static final long serialVersionUID = 1L;
	private IProvider provider;
	private ICommand previousCommand;
	private ICommandExecutionStrategy commandExecutionStrategy = new NullCommandExecutionStrategy();
	private boolean executed = false;

	public ACommand(ICommand previousCommand) {
        this.previousCommand = previousCommand;
	}
	
	@Override
	public final ICommandResult exec() {
		return exec(new ConditionalCommandExecutor());
	}
	
	@Override
    public final ICommandResult exec(ICommandExecutor aCommandExecutor) {
		ICommandResult result = prepareCommandResult();
		doExec(aCommandExecutor, result);
		return result;
	}
	
    @Override
    public final ICommandResult rollback(ICommandExecutor aCommandExecutor){
    	ICommandResult result = prepareCommandResult();
    	doRollback(aCommandExecutor, result);
    	return result;
    }
	
	@Override
	final ICommandResult prepareCommandResult() {
		ICommandResult result = getPreviousCommand().prepareCommandResult();
		result.commandStartExecution(this);
		return result;
	}
	
	@Override
	final void doExec(ICommandExecutor aCommandExecutor, ICommandResult result) {
		getPreviousCommand().doExec(aCommandExecutor, result);
		commandExecutionStrategy = new ExecCommandExecutionStrategy(this, result);
		if (aCommandExecutor.executeNext(result)) {
			 aCommandExecutor.addCommandToPool(this);
		 } else {
			 result.commandEndExecution(this);
		 }
	}
	
    @Override
    void doRollback(ICommandExecutor aCommandExecutor, ICommandResult result){
    	commandExecutionStrategy = new RollbackCommandExecutionStrategy(this, result);
    	if (executed) {
//    		previousRollbackResult.commandStartExecution(this);
			aCommandExecutor.addCommandToPool(this);
        } else {
        	result.commandEndExecution(this);
        }
    	getPreviousCommand().doRollback(aCommandExecutor, result);
    }

	@Override
	public final void visit(IProvider provider) {
		this.provider = provider;
	}
	
	@Override
	public final void run() {
		commandExecutionStrategy.execute();
	}
	
	@Override
    public final ICommandResult rollback(){
		return rollback(new ConditionalCommandExecutor());
    }
    
    @Override
    public final ICommandResult rollback(ICommandResult execResult) {
    	return rollback(execResult, new ConditionalCommandExecutor());
    }
    
    @Override
    public final ICommandResult rollback(ICommandResult execResult, ICommandExecutor aCommandExecutor) {
    	if (!execResult.isValid()) {
    		return rollback(aCommandExecutor);
    	}
    	return new CommandResult(); 
    }
    
    protected abstract void result(ICommandResult result);
    
    protected abstract void internalRollBack(ICommandResult rollBackResult);

    protected ICommand getPreviousCommand() {
		if (previousCommand==null) {
			previousCommand = new NullCommand();
		}
		return previousCommand;
	}

	protected IProvider getProvider() {
		if (provider==null) {
			provider = new NullProvider();
		}
		return provider;
	}

	boolean isExecuted() {
		return executed;
	}

	void setExecuted(boolean executed) {
		this.executed = executed;
	}
	
}
